Package: MorphoRegions
Type: Package
Title: Analysis of Regionalization Patterns in Serially Homologous
        Structures
Version: 0.1.0
Authors@R: c(
  person("Amandine", "Gillet", role = c("cre", "aut"),
         email = "gillet.aman@gmail.com", 
         comment = c(ORCID = "0000-0001-9763-6061")),
  person("Katrina", "Jones", role = c("aut"), 
         email = "katrina.jones@manchester.ac.uk",
	       comment = c(ORCID="0000-0003-1088-0497")),
  person("Noah", "Greifer", role = c("aut"), 
         email = "ngreifer@iq.harvard.edu",
	       comment = c(ORCID="0000-0003-3067-7154")),
  person("Stephanie", "Pierce", role = c("aut"),
         email = "spierce@oeb.harvard.edu", 
         comment = c(ORCID = "0000-0003-0717-1841"))
  )
Description: Computes the optimal number of regions (or subdivisions) and 
    their position in serial structures without a priori assumptions and to visualize 
    the results. After reducing data dimensionality with the built-in function for
    data ordination, regions are fitted as segmented linear regressions
    along the serial structure. Every region boundary
    position and increasing number of regions are iteratively fitted and
    the best model (number of regions and boundary positions) is selected
    with an information criterion. This package expands on the previous
    'regions' package (Jones et al., Science 2018) with improved
    computation and more fitting and plotting options.
License: GPL (>= 2)
Encoding: UTF-8
Depends: R (>= 4.0.0)
Imports: stats, parallel, utils, grDevices, RColorBrewer (>= 1.1-3),
        cluster (>= 2.1.4), scales (>= 1.3.0), ggplot2 (>= 3.5.1), chk
        (>= 0.9.0), pbapply (>= 1.7-2)
Suggests: viridisLite, patchwork (>= 1.1.0), knitr, rmarkdown
VignetteBuilder: knitr
LazyData: true
RoxygenNote: 7.3.2
URL: https://aagillet.github.io/MorphoRegions/,
        https://github.com/aagillet/MorphoRegions
BugReports: https://github.com/aagillet/MorphoRegions/issues
NeedsCompilation: no
Packaged: 2024-08-16 09:17:48 UTC; Amandine Gillet
Author: Amandine Gillet [cre, aut] (<https://orcid.org/0000-0001-9763-6061>),
  Katrina Jones [aut] (<https://orcid.org/0000-0003-1088-0497>),
  Noah Greifer [aut] (<https://orcid.org/0000-0003-3067-7154>),
  Stephanie Pierce [aut] (<https://orcid.org/0000-0003-0717-1841>)
Maintainer: Amandine Gillet <gillet.aman@gmail.com>
Repository: CRAN
Date/Publication: 2024-08-21 12:40:06 UTC
Built: R 4.6.0; ; 2026-04-26 08:56:21 UTC; unix
