# Generated by roxygen2: do not edit by hand

S3method(modelperf,regions_pco)
S3method(modelperf,regions_results_single)
S3method(modelperf,regions_sim)
S3method(plot,regions_pco)
S3method(plot,regions_pco_load)
S3method(plot,regions_pco_select)
S3method(plot,regions_results_single)
S3method(plot,regions_sim)
S3method(plotsegreg,regions_pco)
S3method(plotsegreg,regions_results_single)
S3method(plotsegreg,regions_sim)
S3method(print,regions_BPvar)
S3method(print,regions_modelselect)
S3method(print,regions_modelsupport)
S3method(print,regions_pco)
S3method(print,regions_pco_load)
S3method(print,regions_pco_select)
S3method(print,regions_perf)
S3method(print,regions_results)
S3method(print,regions_results_single)
S3method(print,regions_sim)
S3method(print,summary.regions_pco_select)
S3method(print,summary.regions_results)
S3method(summary,regions_pco_select)
S3method(summary,regions_results)
export(PCOload)
export(PCOselect)
export(addregions)
export(calcBPvar)
export(calcmodel)
export(calcregions)
export(modelperf)
export(modelselect)
export(modelsupport)
export(ncombos)
export(plotsegreg)
export(plotvertmap)
export(process_PC)
export(process_gmPC)
export(process_measurements)
export(simregions)
export(subsample)
export(svdPCO)
import(ggplot2)
import(stats)
